'''
Filter to implement a transformation of the INPUT IMAGE in a 3D-like wireframe
..............................................................................

Author: Diego Nassetti
Version Author: jazzon of gimpchat.com forums <code.maintainer@gmail.com>

Version 1.1.4

Release 1.1.4  Code Clean up, or rather converted to my style.
               Pygimp vaguery of x is True vs x == True addressed.  All "is"
                 comparators converted to "==" to compensate for abnormal
                 Python behavior.
               Some pdb calls replaced with class functions which do the same.

release 1.1.3
               Force new layers to go to top of image if flattening is
                 True, thus they flatten correctly.
               Veried Truth idiosyncracy test was covered.

Release 1.1.2  
               Modified format of the layer merging operation in the
                 ancillary plugin.  Plugin now uses a parasite to carry
                 in a prefix code to apply to the layer names.
               Parasite is removed on succesful completion, and through
                 gimp undo operation.
               Modified the main plugin to use this parasitic data for
                 layer name prefixes.  This makes null and void the
                 changes in 1.1.1 to the layer names.
               Cleaned up the code somewhat.
               Modified the primary plugin to work on a new image, instead of
                 in place.  (Speed / memory optimization)
               Transfer layers back to source image upon completion, then
                 adjust their parent and location and mode as needed.
               Tested against:                   (All Tests Passed)
                 Image with one layer, no groups.
                 Image with many layers, no groups, top layer selected.
                 Image with many layers, no groups, bottom layer selected.
                 Image with many layers, no groups, intermediate layer
                   selected.
                 All above tests inside a group, in an image with many groups.
                 A 66MB image.
                 A 2.4 GB image.

Release 1.1.1 Renamed all layers to be plugin specific names.  They were
                clashing with other similar plugins in this series.

Version 1.1  Modification to Allow Using Layers in Layer Groups
             Creates a copy of the source layer as a base layer.
             Moves the new source layer out of any Layer Groups.
             Completes it's run normally.  (See Flow below)
             Moves the end result layers back into the original Layer Group of
               the initial inDrawable.
             Prefixed the pdb registered name with jaz_ to avoid conflict with
               the original by Diego and dinasset.
             Prefixed primary function with jaz_
             Reworked registration to use my format, and altered register to
               match.
             Removed reliance on Diegos mod_LCS plugin by incorporating it's
               source code here.  Core plugin still prefers the plugin
               over this inclussion.
             Added ancillary function which is API identical, but ignores
               "flatten" option.  This varient auto flattens only the work
               layers.


Release 1.0   initial

Flow implemented to get the final result:

1. Apply adjustments to Luminosity/Contrast/Saturation and save into a buffer
2. Duplicate the layer
3. Apply G'MIC Garagecoder Emboss on this new layer
4. Color-to-Alpha black
5. Add a black layer below and merge
6. Apply G'MIC Wiremap to the input drawable
7. Color-to-Alpha black
8. Add a black layer below and merge

10.If the user requested golden shine...
11....overlay both layers (aftyer selecting only white pixels) with Gold
   texture
12.If the user requested colour, add a new layer on top...
13....and Fill it with the saved layer after step1

----------------------------------------------------------------
COPYRIGHT NOTICE
----------------------------------------------------------------

This program is free software you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program if not, you can view the GNU General Public
License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
Ave, Cambridge, MA 02139, USA.

-------------------------------------------------------------------
other info
-------------------------------------------------------------------

'''
#!/usr/bin/env python
from gimpfu import *
import os
import sys
import random
DEBUG = False
#
#
# ######################
# Registration Variables
# ######################
menu_name =        N_("WireframeMap GC-II(GS)")
author =           "Jazzon of gimpchat.com forums"
author_email =     "code.maintainer@gmail.com"
pub_date =         N_("February 4, 2016")
copyright_holder = "Diego Nassetti"
copyright_email =  ""
copyright_date =   "January, 2016"
blurb =            N_("Derive a 2d wireframe texture from an image. "
                      "Layer Group Safe.")
detail =           N_("This script takes an input image and transform it into "
                      "a 2 dimensional wireframe texture.  The work is done "
                      "in a separate image, and the resulting layers copied "
                      "back to the source image, in their correct place.  "
                      "This is Layer Group safe.")
img_types =        "*"
menu_parent =      "<Image>/"
menu_slot =        "Filters/Render/Wireframe"
pdb_name =         "jaz_wireframe_gc_ii"
parameters = [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Net square parm", 100, (50, 200, 10)),
      (PF_TOGGLE, "golden", "Add golden shine to wireframe?", False),
      (PF_TOGGLE, "colours", "Restore colours to image?", False),
      (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ]
returns = []


# Source was the DIEGO_COLORS_Mod_LCS.py file of the plugin mod_LCS
# by dinaset.  License etc in the comments for mod_lcs function below.
def get_saturation(layer):
    """Service Function by Saul Goode.  See documentation for mod_LCS."""

    image = pdb.gimp_item_get_image(layer)
    sat_image = pdb.plug_in_decompose_registered(image, layer,
                                                 "Saturation", False )
    # decompose returns an array, in this case of 1 element
    # element 0 is the only one returned
    sat_layer = pdb.gimp_image_get_active_layer(sat_image[0])
    saturation = pdb.gimp_histogram(sat_layer, HISTOGRAM_VALUE, 0, 255 )
    pdb.gimp_image_delete(sat_image[0])
    return (saturation[0])
#
#
# #############################################
# Diego's DIEGO_COLORS_Mod_LCS plugin contents.
# #############################################
# Source was the DIEGO_COLORS_Mod_LCS.py file of the plugin mod_LCS
# by dinaset.  License etc in the comments for mod_lcs function below.
def get_saturation(layer):
    """Service Function by Saul Goode.  See documentation for mod_LCS."""

    image = pdb.gimp_item_get_image(layer)
    sat_image = pdb.plug_in_decompose_registered(image, layer,
                                                 "Saturation", False )
    # decompose returns an array, in this case of 1 element
    # element 0 is the only one returned
    sat_layer = pdb.gimp_image_get_active_layer(sat_image[0])
    saturation = pdb.gimp_histogram(sat_layer, HISTOGRAM_VALUE, 0, 255 )
    pdb.gimp_image_delete(sat_image[0])
    return(saturation[0])
#
#
# #############################################
# Diego's DIEGO_COLORS_Mod_LCS plugin contents.
# #############################################
# Source was the DIEGO_COLORS_Mod_LCS.py file of the plugin mod_LCS
# by dinaset. Register call for said gimp plugin contained these lines
# for copyright info, with the additional material in triple quotes below.
#   "Diego Nassetti ",
#   "December 2015",
# =============================================================================
# 
def mod_LCS(inImage, inLayer, action, strength, target_L, target_C, target_S):
    """
    ========================================================================
    Filter to implement an automatic or semi_automatic adjustment of:
    Luminosity, Contrast, Saturation
    ______________________________________________________________________
    0.1.0-f  This function copy placed into this code by jazzon of gimpchat.com
    0.1.0    initial version

    note: it includes a service function provided by Saul Goode (thanks!)
          to detect the saturation level

    Flow implemented to get the final result:

    0.  Analyse action requested:

    A.  If action = Info only:

    A1. get from the histogram: the Mean Value, the Std Deviation, the
         Saturation level
    A2. compare them with the targets and calculate the deltas
    A3. according to these deltas classify:
          the current Luminosity: Low, Average, High
          the current Contrast: Low, Average, High
          the current Saturation: Low, Average, High
    note: Average means around the target +/_ 32
    A4. display to the user the calculated classes

    B.  action = Enhance:

    B1. analyse the level of strength requested for the adjustments
    B2. if not "auto", all factors are equally set to the level requested
    B3. if "auto" each single factor is set according to the current class:
    B4.  weak for a low class
         normal for an average class
         weak for a high class
    B5. calculate the current Mean Value from the histogram
    B6. calculate the adjusting Set Value
    B7. apply it, using the gimp_levels native filter
    B8. calculate the current Contrast from the histogram (std deviation)
    B9. calculate the adjusting Set Value
    B10.apply it, using the gimp_brightess_contrast native filter
    B11.calculate the current Saturation from Saul Goode's service routine
    B12.calculate the adjusting Set Value
    B13.apply it, using the gimp_hue_saturation native filter
    ________________________________________________________________
    ________________________________________________________________

    This program is free software# you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation# either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY# without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program# if not, you can view the GNU General Public
    License version 3 at the web site http://www.gnu.org/licenses/gpl_3.0.html
    Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
    Ave, Cambridge, MA 02139, USA."""

# Initialize an undo, so the process can be undone with a single undo
    pdb.gimp_image_undo_group_start(inImage)

# set-up constant values
    LowL = 0
    AvgL = 1
    HighL = 2
    LowC = 0
    AvgC = 1
    HighC = 2
    LowS = 0
    AvgS = 1
    HighS = 2
    LCSclass = ("Low", "Average", "High")
    RfactorL = 9
    RfactorC = 9
    RfactorS  = 9
    Luminosity = 9
    Contrast = 9
    Saturation = 9

    (meanValue, stdDev,
     median, pixels,
     count, percent) = pdb.gimp_histogram(inLayer, HISTOGRAM_VALUE, 0 , 255)

    meanSat = get_saturation(inLayer)  # mean of values

    if meanValue < (target_L - 32):
        Luminosity = LowL
    elif meanValue > (target_L + 32):
        Luminosity = HighL
    else:
        Luminosity = AvgL

    if stdDev < (target_C - 32):
        Contrast = LowC
    elif stdDev > (target_C + 32):
        Contrast = HighC
    else:
        Contrast = AvgC

    if meanSat < (target_S - 32):
        Saturation = LowS
    elif meanSat > (target_S + 32):
        Saturation = HighS
    else:
        Saturation = AvgS

 # perform requested action

    if action == 1:    # logic for info only
        pdb.gimp_message("mean value: " + str(meanValue))
        pdb.gimp_message("std deviation: " + str(stdDev))
        pdb.gimp_message("mean saturation: " + str(meanSat))
        pdb.gimp_message("Luminosity class: " + LCSclass[Luminosity])
        pdb.gimp_message("Contrast class: " + LCSclass[Contrast])
        pdb.gimp_message("Saturation class: " + LCSclass[Saturation])

 # end if action = info only
 # =========================

 # logic for auto enhance start here
 # =================================

    if action == 0:    # logic for enhance

 # apply choosen strength

        if strength > 0:              # means: not auto
            if strength == 3:         # request for weak adjustment
                RfactorL = 3
            elif strength == 1:       # request for strong adjustmnet
                RfactorL = 1
            else:                     # request for normal adjustment
                RfactorL = 2

            RfactorC = RfactorL       # all 3 equal
            RfactorS = RfactorL       # all 3 equal

        else:                         # auto adjustment <======================
  # auto is based on current status

            if Luminosity == HighL:
                RfactorL = 3          # weak against high
            elif Luminosity == LowL:
                RfactorL = 3          # weak against low
            else:
                RfactorL = 2          # normal against average

            if Contrast == HighC:
                RfactorC = 3          # weak against high
            elif Contrast == LowC:
                RfactorC = 3          # weak against low
            else:
                RfactorC = 2          # normal against average

            if Saturation == HighS:
                RfactorS = 3          # weak against high
            elif Saturation == LowS:
                RfactorS = 3          # weak against low
            else:
                RfactorS = 2          # normal against average

        #pdb.gimp_message("RfL="+str(RfactorL)+",
        #    RfC="+str(RfactorC)+", RfS="+str(RfactorS))

# mean value

        # avoid division by zero

        if meanValue < 0.1:            # check zero
            meanValue = 0.1

        setMeanValue = target_L / meanValue  # the ratio desired/actual
        # reduce the ratio
        setMeanValue = (setMeanValue + RfactorL) / (RfactorL + 1)
        setMeanValue = setMeanValue * setMeanValue  # power of the ratio
        if setMeanValue < 0.1:        # check range: must be between 0.1 and 10
            setMeanValue = 0.1
        elif setMeanValue > 10:
            setMeanValue = 10

        #pdb.gimp_message("set midpoint: "+str (setMeanValue))

        pdb.gimp_levels(inLayer, HISTOGRAM_VALUE, 0, 255, setMeanValue, 0, 255)
        (newmeanValue, newstdDev,
         median, pixels, count,
         percent) = pdb.gimp_histogram(inLayer, HISTOGRAM_VALUE, 0 , 255)
        #pdb.gimp_message ("new mean value: "+str(newmeanValue))

# standard deviation
        setContrast = target_C - stdDev                        # the difference
        setContrast = setContrast / RfactorC            # reduce the difference
        if setContrast < -127:      # check range: must be between -127 and 127
            setContrast = -127
        elif setContrast > 127:
            setContrast = 127

        #pdb.gimp_message ("delta contrast: "+str(setContrast))

        pdb.gimp_brightness_contrast(inLayer, 0, setContrast)
        (newmeanValue, newstdDev,
         median, pixels, count,
         percent) = pdb.gimp_histogram(inLayer, HISTOGRAM_VALUE, 0 , 255)
        #pdb.gimp_message ("new mean value: "+str(newmeanValue))
        #pdb.gimp_message ("new std deviation: "+str(newstdDev))

# mean saturation
        # actual sat (new, because may have been changed)
        newmeanSat = get_saturation(inLayer)
        setSaturation = target_S - newmeanSat      # the difference (using new)
        setSaturation = setSaturation / RfactorS        # reduce the difference
        setSaturation = setSaturation * 100 / 128           # set as percentage
        if setSaturation < -100:    # check range: must be between -100 and 100
            setSaturation = -100
        elif setSaturation > 100:
            setSaturation = 100

        #pdb.gimp_message ("delta saturation: "+str(setSaturation))

        pdb.gimp_hue_saturation(inLayer, 0, 0, 0, setSaturation)
        newmeanSat = get_saturation(inLayer)
        #pdb.gimp_message ("new mean saturation: "+str(meanSat))

    pdb.gimp_displays_flush

    pdb.gimp_image_undo_group_end(inImage)

    return
#
#
###############################################################################
#                      Primary Plugin Code
###############################################################################
def jaz_plug_code(inImage, inDrawable, inSize, doGold, doColour, doFlatten) :
    if DEBUG:
        print "Entering main script with:"
        print inImage, inDrawable, inSize, doColour, doFlatten

    # ########################
    # Set up initial variables
    # ########################
    width = inImage.width
    height = inImage.height
    target_parent = inDrawable.parent
    target_name = inDrawable.name
    typ = inImage.base_type
#
#
    # #####################################
    # Save Context and Set up an undo group
    # #####################################
    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    if not DEBUG:
        pdb.gimp_image_undo_group_start(inImage)
#
#
    # ###############################
    # Configure the layer name prefix
    # ###############################
    parasiteName = "jaz_wfm_gc2"
    if parasiteName in inImage.parasite_list():
        pfx = str(inImage.parasite_find(parasiteName))
    else:
        choice = ("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                  "abcdefghijklmnopqrstuvwxyz")
        pfx = ''.join(random.choice(choice) for i in range(4))
        
    if DEBUG:
        print "Prefix is: ", pfx
#
#
    # ###############################
    # Create the layer / buffer names
    # ###############################
    # Variables are shorter than the command structure to
    # create the variable names using the pfx. (Convieniance)
    inDrawName =    "_".join((pfx, "Origin copy"))
    origBuffName =  "_".join((pfx, "myBufferName1"))
    embosLayName =  "_".join((pfx, "Embosssed Layer"))
    #gridLayName =   "_".join((pfx, "Grid Layer"))
    blackLay1Name = "_".join((pfx, "Black Layer 1"))
    blackLay2Name = "_".join((pfx, "Black Layer 2"))
    savedLayName =  "_".join((pfx, "Saved Layer"))
#
#
    # ################################################
    # Create the new image where the work will be done
    # ################################################
    oldImage = inImage
    floating = pdb.gimp_edit_copy(inDrawable)
    inImage = pdb.gimp_edit_paste_as_new()
    if not DEBUG:  # No undo for this image unless debugging
        inImage.disable_undo()

    if DEBUG:
        print "inImage, layers: ", inImage, inImage.layers

    inDrawable = inImage.layers[0]
    inDrawable = inDrawable.copy()  # make a copy so original is untouched
    inImage.add_layer(inDrawable, 0)
    inDrawable.name = inDrawName
#
#
    # ###############################
    # Add an Alpha Channel if missing
    # ###############################
    alpha = pdb.gimp_drawable_has_alpha(inDrawable)
    if alpha == 0:
        pdb.gimp_layer_add_alpha(inDrawable)
        if DEBUG:
            print "Added an alpha channel to inDrawable"
#
#
    # ####################################
    # Adjust the saturation/value/contrast
    # ####################################
    # Determine if the mod is installed
    # and use that if present, else use
    # the internal code.
    test = False
    if not DEBUG:
        for e in dir(gimp.pdb):
            f = e.lower()
            if "mod_lcs" in f and "python" in f:
                test = True
                break
    
    if DEBUG:
        print "Forcing internal mod_lcs"
        print "Was it actually detected? ", test
            
    if test:
        pdb.python_fu_mod_LCS(inImage, inDrawable, 0, 3, 128, 64, 152)
    else:            
        mod_LCS(inImage, inDrawable, 0, 3, 128, 64, 152)
#
#
    # #######################################
    # preserve the original layer as a buffer
    # #######################################
    origBufferName = pdb.gimp_edit_named_copy(inDrawable, origBuffName)
    if DEBUG:
        print "Buffer Saved as ", origBuffName
#
#
    # ##########################################
    # Create the emboss layer, and run the G'MIC
    # emboss script End by rendering Black
    # transparent in the layer.
    # ##########################################
    EmbossLayer = inDrawable.copy()
    EmbossLayer.mode = NORMAL_MODE
    EmbossLayer.name = embosLayName
    EmbossLayer.opacity = 100.0
    inImage.add_layer(EmbossLayer, 0)
    EmbossLayer = inImage.active_layer
    # [G'MIC] Emboss: -gcd_emboss 128,0
    # 1 below = input only active layer
    pdb.plug_in_gmic(inImage, EmbossLayer, 1, "-v - -gcd_emboss 128,0")
    pdb.plug_in_colortoalpha(inImage, EmbossLayer, (0, 0, 0))
#
#
    # ##################################
    # Create the BlackLayer1 as a base
    # for the EmbossLayer to merge with.
    # ##################################
    BlackLayer1 = pdb.gimp_layer_new(
        inImage, width, height, typ, blackLay1Name, 100.00, NORMAL_MODE)

    inImage.add_layer(BlackLayer1, 1)
    pdb.gimp_context_set_background((0, 0, 0))
    pdb.gimp_drawable_fill(BlackLayer1, BACKGROUND_FILL)
#
#
    # #########################################
    # Merge the EmbossLayer with BlackLayer1
    # to create mergedLayer1 and set it's alpha
    # to grey value 96.
    # #########################################
    mergedLayer1 = pdb.gimp_image_merge_down(
        inImage, EmbossLayer, CLIP_TO_BOTTOM_LAYER)

    pdb.plug_in_colortoalpha(inImage, mergedLayer1, (96, 96, 96))
#
#
    # ################################
    # Process gold tinting of the grid
    # if user requested it.  Otherwise
    # alter contrast to improve
    # visibility.
    # ################################
    if doGold == True :
        pdb.gimp_context_set_pattern("Gold 322")
        pdb.gimp_image_select_color(
            inImage, CHANNEL_OP_REPLACE, mergedLayer1, (255, 255, 255))
        pdb.gimp_edit_fill(mergedLayer1, PATTERN_FILL)
        pdb.gimp_selection_none(inImage)
        pdb.gimp_brightness_contrast(mergedLayer1, -50, 0)
    else :
        pdb.gimp_brightness_contrast(mergedLayer1, -90, 0)
#
#
    # ####################################
    # Prepare source layer to be merged by
    # wiremapping it with G'MIC, and
    # setting it's alpha channel to the
    # color black.
    # ####################################
    pdb.gimp_image_set_active_layer(inImage, inDrawable)
    # [G'MIC] Wiremap: -gcd_wiremap 100,300,0.3
    cmd = "".join(("-v - -gcd_wiremap ", str(inSize), ",300,0.5,0.3,0"))
    # 1 below = input only active layer
    pdb.plug_in_gmic(inImage, inDrawable, 1, cmd)
    pdb.plug_in_colortoalpha(inImage, inDrawable, (0, 0, 0))
#
#
    # ###################################
    # Create the BlackLayer2 as a base
    # for the source layer to merge with.
    # ###################################
    BlackLayer2 = pdb.gimp_layer_new(
        inImage, width, height, typ, blackLay2Name, 100.00, NORMAL_MODE)

    inImage.add_layer(BlackLayer2, 2)
    pdb.gimp_context_set_background((0, 0, 0))
    pdb.gimp_edit_fill(BlackLayer2, BACKGROUND_FILL)
#
#
    # #########################################
    # Create mergedLayer2 by merging the source
    # down, into BlackLayer2, and adjust alpha
    # to grey color 40.
    # #########################################
    mergedLayer2 = pdb.gimp_image_merge_down(
        inImage, inDrawable, CLIP_TO_BOTTOM_LAYER)
    pdb.plug_in_colortoalpha(inImage, mergedLayer2, (40, 40, 40))
#
#
    # ################################
    # Process second step of the gold
    # tinting of the grid if user 
    # requested it.
    # ################################
    if doGold == True:
        if (width + height) > 3000:
            pdb.plug_in_dilate(inImage, mergedLayer2, 0, 0, 1.0, 15, 0, 255)
        if (width + height) > 5000:
            pdb.plug_in_dilate(inImage, mergedLayer2, 0, 0, 1.0, 15, 0, 255)
        pdb.gimp_image_select_color(
            inImage, CHANNEL_OP_REPLACE, mergedLayer2, (255, 255, 255))
        pdb.gimp_context_set_pattern("Gold 322")
        pdb.gimp_edit_fill(mergedLayer2, PATTERN_FILL)
        pdb.gimp_selection_none(inImage)
#
#
    # ####################################
    # Process restoring color to the image
    # if user requested it.
    # ####################################
    if doColour == True:
        if DEBUG:
            print "Processing do color"

        SavedLayer = pdb.gimp_layer_new(
            inImage, width, height, typ, savedLayName, 100.00, COLOR_MODE)
        if DEBUG:
            print "SavedLayer: ", SavedLayer
            
        inImage.add_layer(SavedLayer, 0)
        pdb.gimp_floating_sel_anchor(
            pdb.gimp_edit_named_paste(SavedLayer, origBufferName, False))
        pdb.plug_in_gauss_iir2(
            inImage, SavedLayer, int(width / 70), int(height / 70))
#
#
    # ################################################
    # Prepare service variables for use in placing the
    # layers back in the correct image, and in the
    # correct group (if any).
    # ################################################
    lst = list(inImage.layers)
    lst.reverse()
    # Remove the layer that was copied in from
    # the list so it isn't dubled in the final
    # result.
    inImage.remove_layer(lst.pop(0))
    # new_layer() -> Undocumented use.  Creates a
    # transparent layer with image dimensions, and
    # inserts it into the image all at once.
    # t=target for pastes, although due to method is
    # still empty when done.
    t = oldImage.new_layer()
    t.name = "_".join((pfx, "trash_layer"))
    # Target_name = the name of the source layer in
    # original image.  It's position is the base for
    # where new layers go (each goes 1 above it) as
    # they are copied back.
    t_PosHolder = pdb.gimp_image_get_layer_by_name(oldImage, target_name)
    target_position = pdb.gimp_image_get_item_position(oldImage, t_PosHolder)
    ct = 0
#
#
    # #####################################
    # Copy the layers over, and put them in
    # the right place and group.
    # #####################################
    while len(lst) > 0:
        s = lst.pop(0)  # s = source
        pdb.gimp_edit_copy(s)
        l = pdb.gimp_edit_paste(t, True)  # l = layer, t = paste target
        l.name = s.name
        l.mode = s.mode  # <- IMPORTANT gimp loses this in process!
        pos = target_position - ct - 1  # position in local stack
        pos = pos if pos >= 0 else 0  # Ensure non negative
        if doFlatten == True:
            # Layer goes at top of visible stack, and outside
            # any group if flattening.
            pos = 0
            target_parent = None
            
        pdb.gimp_image_reorder_item(oldImage, l, target_parent, pos)
        
    oldImage.remove_layer(t)
#
#
    # ##############################
    # Handle flattening of the image
    #  if so requested
    # ##############################
    if doFlatten == True:
        oldImage.flatten()
#
#
    # #####################################
    # Restore previous environment and exit
    # #####################################
    if not DEBUG:
        inImage.enable_undo()
        gimp.delete(inImage)
        pdb.gimp_image_undo_group_end(oldImage)

    pdb.gimp_context_pop
    pdb.gimp_context_set_background(currBG)
    pdb.gimp_context_set_foreground(currFG)
    print "jaz_wireframe_gc_ii completed succesfully"
    if DEBUG:
        pdb.gimp_display_new(inImage)

    gimp.displays_flush()
    
    return
#
#
# ###################################
# 1.1 Added secondary pdb function.
# Registration params at end of file.
# ###################################
def jaz_plug_code_2(inImage, inDrawable, inSize, doGold, doColour) :

    if DEBUG:
        print "In jaz_plug_code_2 with: ",inImage, inDrawable, inSize, \
              doGold, doColour

    if not DEBUG:
        pdb.gimp_image_undo_group_start(inImage)
#
#
    # ##############################################
    # Set up a parasite for carrying in a new prefix
    # ##############################################
    parasiteName = "jaz_wfm_gc2"
    cnt, lst = pdb.gimp_image_get_parasite_list(inImage)
    if parasiteName in lst:
        inImage.parasite_detach(parasiteName)

    choice = ("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
              "abcdefghijklmnopqrstuvwxyz")
    pfx = ''.join(random.choice(choice) for i in range(4))
    inImage.attach_new_parasite(parasiteName, 3, pfx)
#
#
    # ####################
    # Call the main plugin
    # ####################
    pdb.python_fu_jaz_wireframe_gc_ii(inImage, inDrawable, inSize, doGold,
                                      doColour, False)
#
#
    # ##########################
    # Merge the work layers down
    # ##########################
    lst = ["_".join((pfx, "Saved Layer")),
           "_".join((pfx, "Black Layer 1")),
           "_".join((pfx, "Black Layer 2")) ]
    if DEBUG:
        print "lst: ", lst

    if doColour == False:
        lst.pop(0)
        if DEBUG:
            print "lst: ", lst

    nextL = pdb.gimp_image_get_layer_by_name(inImage, lst.pop(0))
    inImage.active_layer = nextL
    while len(lst) > 0:
        __ =  pdb.gimp_image_merge_down(inImage, nextL, CLIP_TO_BOTTOM_LAYER)
        nextL = pdb.gimp_image_get_layer_by_name(inImage, lst.pop(0))
        inImage.active_layer = nextL

    nextL.name = "Wireframe Map Garagecoder II"
#
#
    # #################
    # Clean up and exit
    # #################
    inImage.parasite_detach(parasiteName)
    if not DEBUG:
        pdb.gimp_image_undo_group_end(inImage)

    print "jaz_wireframe_gcii_ac completed succesfully"
    return
#
#
# ####################################
# Registration Primary with Gimp's PDB
# ####################################
# No need to edit below here
author =    "".join((author, " <", author_email, ">"))
copyright = "".join((copyright_holder, " <", copyright_email, "< ", pub_date))
# Where is a tuple
where =     ("\n", N_("File Location"), ": ", str(os.path.abspath(sys.argv[0])))
where_i_am = "".join(where)
# Who is a tuple
who = ( "\n", N_("Menu Location"), ": ", menu_slot, menu_name )
who_i_am = ''.join(who)
register(
    pdb_name, blurb, detail, author, copyright, pub_date, menu_name, img_types,
    parameters, returns, jaz_plug_code, menu="".join((menu_parent, menu_slot)))
#
#
# ########################################
# Registration Variables for second plugin
# ########################################
menu_name =        N_("WireframeMap GCII(GS-AC)")
blurb =            N_("Derive a 2d wireframe texture from an image.\n "
                      "Group Safe with auto collapsed work layers.")
detail =           N_("This script takes an input image and transform it into "
                      "a 2 dimensional wireframe texture.  The work is "
                      "performed on a separate image, and the resultant "
                      "layers copied back to original. Then those layers "
                      "only are flattened, to the position occupied by "
                      "the original source layer.")
pdb_name =         "jaz_wireframe_gc_ii_ac"
parameters = [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Net square parm", 100, (50, 200, 10)),
      (PF_TOGGLE, "golden", "Add golden shine to wireframe?", False),
      (PF_TOGGLE, "colours", "NOTE: This does not work as You might expect "
          "with auto-collapse.\nRestore colours to image?", False),
    ]
#
#
# #########################################
# Registration of Secondary with Gimp's PDB
# #########################################
author =    "".join((author, " <", author_email, ">"))
copyright = "".join((copyright_holder, " <", copyright_email, "< ", pub_date))
# Where is a tuple
where =     ("\n", N_("File Location"), ": ", str(os.path.abspath(sys.argv[0])))
where_i_am = "".join(where)
# Who is a tuple
who = ( "\n", N_("Menu Location"), ": ", menu_slot, menu_name )
who_i_am = ''.join(who)
register(
    pdb_name, blurb, detail, author, copyright, pub_date, menu_name, img_types,
    parameters, returns, jaz_plug_code_2, menu="".join((menu_parent, menu_slot)))

main()
